<?php

/**
 * @file
 * Rules integration for amoCRM.
 */

/**
 * @return bool|\RetailCrm\ApiClient
 */
function retailcrm_get_api_client() {
  if (FALSE === variable_get('retailcrm_url', FALSE)
    || FALSE === variable_get('retailcrm_api_key', FALSE)) {
    return FALSE;
  }
  return new \RetailCrm\ApiClient(
    variable_get('retailcrm_url'),
    variable_get('retailcrm_api_key'),
    \RetailCrm\ApiClient::V5
  );
}

/**
 * Implements hook_rules_action_info().
 *
 * @ingroup rules
 */
function retailcrm_rules_action_info() {
  $actions = array();
  try {
    if (!($client = retailcrm_get_api_client())) {
      throw new \Exception(t('Cannot initialize RetailCRM client. Rules actions will not be defined.'));
    }
    unset($client);
  } catch (\Throwable $e) {
    watchdog_exception('RetailCRM', $e);
  }
  $actions['retailcrm_rules_action_sync_user_to_crm_customer'] = \Drupal\retailcrm\Plugin\RulesAction\SyncUserCustomerAction::info();
  $actions['retailcrm_rules_action_sync_order_to_crm_order'] = \Drupal\retailcrm\Plugin\RulesAction\SyncOrderToCrmOrderAction::info();

  return $actions;
}

function retailcrm_rules_action_sync_user_to_crm_customer($user) {
  return \Drupal\retailcrm\Plugin\RulesAction\SyncUserCustomerAction::execute($user);
}

function retailcrm_rules_action_sync_order_to_crm_order($user, $order) {
  return \Drupal\retailcrm\Plugin\RulesAction\SyncOrderToCrmOrderAction::execute($user, $order);
}
