<?php

namespace Drupal\retailcrm\Plugin\RulesAction;

class SyncOrderToCrmOrderAction {

  public static function info() {
    return array(
      'label' => t('Syncronize Order with CRM order'),
      'group' => t('RetailCRM'),
      'base' => 'retailcrm_rules_action_sync_order_to_crm_order',
      'parameter' => array(
        'user' => array('type' => 'user', 'label' => t('User')),
        'commerce_order' => array(
          'type' => 'commerce_order',
          'label' => t('Order'),
        ),
      ),
    );
  }

  private static function get_order_items($order) {
    $items = array();
    $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
    $product_types = array_keys(commerce_product_types());
    foreach ($order_wrapper->commerce_line_items as $delta => $line_item_wrapper) {
      if (!in_array($line_item_wrapper->getBundle(), $product_types)) {
        continue;
      }
      $product_wrapper = $line_item_wrapper->commerce_product;
      $items[$delta]['offer']['externalId'] = $product_wrapper->product_id->value();
      $items[$delta]['offer']['xmlId'] = $product_wrapper->field_xml_id->value();
      $items[$delta]['initialPrice'] = $line_item_wrapper->commerce_unit_price->amount_decimal->value();
      $items[$delta]['quantity'] = intval($line_item_wrapper->quantity->value());
      $items[$delta]['productName'] = $product_wrapper->title->value();
    }
    drupal_alter('retailcrm_order_items', $items);
    return $items;
  }

  public static function execute($user, $order) {
    $client = retailcrm_get_api_client();

    /**
     * Create or Update order
     */
    $result = $client->request->ordersGet($order->order_id);
    if (!$result->isSuccessful()) {
      // Create new order.
      $order_info = array(
        'status' => retailcrm_order_status($order->status),
        'createdAt' => format_date($order->changed, 'custom', 'Y-m-d H:i:s'),
        'externalId' => $order->order_id,
        'email' => $user->mail,
        'customer' => array(
          'externalId' => $user->uid,
        ),
        'items' => self::get_order_items($order),
      );


      drupal_alter('retailcrm_order', $order_info);
      $result = $client->request->ordersCreate($order_info);
      if (!$result->isSuccessful() || 201 !== $result->getStatusCode()) {
        watchdog('RetailCRM', $result->getErrorMsg(), array(), WATCHDOG_ERROR);
      }
    } else {
      // Update order.
      $response = $result->getResponse();
      $order_info = $response['order'];
      module_invoke_all('retailcrm_order', $order_info);
      $order_info['email'] = $user->mail;
      $order_info['status'] = retailcrm_order_status($order->status);
      $order_info['externalId'] = $order->order_id;
      $order_info['customer'] = array(
        'externalId' => $user->uid,
      );
      $order_info['items'] = self::get_order_items($order);
      drupal_alter('retailcrm_order', $order_info);
      $result = $client->request->ordersEdit($order_info);
      if (!$result->isSuccessful() || 200 !== $result->getStatusCode()) {
        watchdog('RetailCRM', $result->getErrorMsg(), array(), WATCHDOG_ERROR);
      }
    }

    // Create payment.
    if (isset($order->data['commerce_payment_order_paid_in_full_invoked']) && $order->data['commerce_payment_order_paid_in_full_invoked'] === TRUE) {
      $transactions = commerce_payment_transaction_load_multiple(array(), array('order_id' => $order->order_id));
      if (!empty($transactions)) {
        foreach ($transactions as $transaction) {
          $payment_id = FALSE;
          if (!empty($order_info['payments'])) {
            foreach ($order_info['payments'] as $payment_info) {
              if (isset($payment_info['externalId']) && $payment_info['externalId'] === $order->order_id) {
                $payment_id = $payment_info['id'];
                break;
              }
            }
          }
          $payment_info = array(
            'externalId' => $transaction->transaction_id,
            'amount' => $transaction->amount / 100,
            'paidAt' => format_date($transaction->changed, 'custom', 'Y-m-d H:i:s'),
            'comment' => t(str_ireplace(array("<br />","<br>","<br/>"), "\r\n", $transaction->message), $transaction->message_variables),
            'order' => array('externalId' => $transaction->order_id),
            'type' => retailcrm_payment_type($transaction->payment_method),
            'status' => retailcrm_payment_status($transaction->status),
          );
          if ($payment_id) {
            $payment_info['id'] = $payment_id;
            $result = $client->request->ordersPaymentEdit($payment_info);
            if (!$result->isSuccessful() || 200 !== $result->getStatusCode()) {
              watchdog('RetailCRM', $result->getErrorMsg(), array(), WATCHDOG_ERROR);
            }
          } else {
            $result = $client->request->ordersPaymentCreate($payment_info);
            if (!$result->isSuccessful() || 201 !== $result->getStatusCode()) {
              watchdog('RetailCRM', $result->getErrorMsg(), array(), WATCHDOG_ERROR);
            }
          }
        }
      }
    }

    return TRUE;
  }

}