<?php

namespace Drupal\retailcrm\Plugin\RulesAction;

class SyncUserCustomerAction {

  public static function info() {
    return array(
      'label' => t('Syncronize User with CRM customer'),
      'group' => t('RetailCRM'),
      'base' => 'retailcrm_rules_action_sync_user_to_crm_customer',
      'parameter' => array(
        'user' => array('type' => 'user', 'label' => t('User')),
      ),
      'provides' => array(
        'user' => array('type' => 'user', 'label' => t('User'), 'save' => TRUE),
      ),
    );
  }

  private static function create_customer($user) {
    if (!isset($user->uid)) {
      return FALSE;
    }
    $client = retailcrm_get_api_client();

    $customer_info = array(
      'externalId' => $user->uid,
      'createdAt' => format_date($user->created, 'custom', 'Y-m-d H:i:s'),
      'contragent' => array('contragentType' => 'individual'),
      'email' => $user->mail,
    );
    drupal_alter('retailcrm_customer', $customer_info);
    try {
      $result = $client->request->customersCreate($customer_info);
    } catch (\Throwable $e) {
      watchdog_exception('RetailCRM', $e);
    }
    if ($result->isSuccessful() && 201 === $result->getStatusCode()) {
      return $result->id;
    }
    return FALSE;
  }

  private static function update_customer($customer_info) {
    if (empty($customer_info)) {
      return FALSE;
    }

    // check data is altered.
    $checksize = strlen(serialize($customer_info));
    drupal_alter('retailcrm_customer', $customer_info);
    if ($checksize === strlen(serialize($customer_info))) {
      return $customer_info['id'];
    }

    $client = retailcrm_get_api_client();
    try {
      $result = $client->request->customersEdit($customer_info);
    } catch (\Throwable $e) {
      watchdog_exception('RetailCRM', $e);
    }
    if ($result->isSuccessful() && 200 === $result->getStatusCode()) {
      return $result->id;
    }
    return FALSE;
  }

  public static function execute($user) {
    if (!isset($user->uid)) {
      return array('user' => $user);
    }
    $client = retailcrm_get_api_client();

    try {
      $result = $client->request->customersGet($user->uid);
    } catch (\Throwable $e) {
      watchdog_exception('RetailCRM', $e);
    }
    if ($result->isSuccessful()) {
      $response = $result->getResponse();
      $customer_info = $response['customer'];
      module_invoke_all('retailcrm_customer', $customer_info);
      $customer_id = self::update_customer($customer_info);
    } else {
      try {
        $result = $client->request->customersList(array('email' => $user->mail));
      } catch (\Throwable $e) {
        watchdog_exception('RetailCRM', $e);
      }
      if ($result->isSuccessful()) {
        $response = $result->getResponse();
        if (!empty($response['customers'])) {
          // customer found.
          $customer_info = reset($response['customers']);
          module_invoke_all('retailcrm_customer', $customer_info);
          $customer_id = self::update_customer($customer_info);
        } else {
          // create new customer.
          $customer_id = self::create_customer($user);
        }
      }
    }
    return array('user' => $user);
  }

}